module  uart_sendout
#(
	parameter baud_cnt_const  =5208   //5208,
)
( 
	input      clk,
	input      rst_n,
	input[7:0] txout,
	input      txout_req,
	output     txout_ack,
	output     txd
);
  

reg        baud_pluse = 0; 
reg[15:0]  baud_cnt  = 0;
always @(posedge clk )
begin
  if(!rst_n)
  begin
    baud_cnt  <=0;
	baud_pluse<=1'b0;
  end
  else 
    if(baud_cnt==baud_cnt_const-1)
	   begin
         baud_cnt  <=0;
		 baud_pluse<=1'b1;
	   end
    else
	  begin
   	    baud_cnt   <= baud_cnt +1'b1;
		baud_pluse <=1'b0;
	  end
end

localparam  idle      = 5'b00001;  
localparam	out_sync  = 5'b00010;
localparam  out_txd   = 5'b00100; 
localparam  out_ack   = 5'b01000; 
localparam  out_over  = 5'b10000; 

reg[4:0]   out_state       = 0; 
reg[10:0]  temp_data       = 0; 
reg[3:0]   out_txd_cnt     = 0;
reg        txd_temp        = 1; 
reg        txout_ack_temp  = 0; 
always @(  posedge clk  )
begin
  if(!rst_n)
  begin
    out_state      <= idle;
	temp_data      <= 0; 
	out_txd_cnt    <= 0; 
	txd_temp       <= 1'b1;
	txout_ack_temp <= 0;
  end
  else

      case (out_state)
        idle:
          begin
			if(txout_req)
				begin
					out_state  <= out_sync; 
					temp_data  <= {2'b11,txout[7:0],1'b0};
				end
    	  end  
		out_sync:
			begin
				if(baud_pluse)	
					out_state <= out_txd;
			end
        out_txd:
			begin
				if(out_txd_cnt==4'd11) //ʮλݷ
				  begin
					out_txd_cnt<= 0;
					out_state  <= out_ack;
					txd_temp   <= 1'b1;
				  end
				else
				  begin
					txd_temp   <= temp_data[0];                			
					if(baud_pluse)
						begin
							temp_data  <={temp_data[0],temp_data[10:1]};	
							out_txd_cnt<= out_txd_cnt+1'b1;    
						end
				  end
			  end
		out_ack://λݲ
			begin
				txout_ack_temp <= 1'b1;
				out_state      <= out_over;	
			end
		out_over:
			begin
				txout_ack_temp <= 1'b0;
				if(baud_pluse)
					out_state      <= idle;	
			end
	    default:
	      begin
	       out_state <=idle;
	       txd_temp  <=1'b1;
	     end
       endcase
end

assign txd = txd_temp;
assign txout_ack  = txout_ack_temp;
endmodule